/* ownCloud Android Library is available under MIT license
 *   Copyright (C) 2019 ownCloud GmbH.
 *
 *   Permission is hereby granted, free of charge, to any person obtaining a copy
 *   of this software and associated documentation files (the "Software"), to deal
 *   in the Software without restriction, including without limitation the rights
 *   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *   copies of the Software, and to permit persons to whom the Software is
 *   furnished to do so, subject to the following conditions:
 *
 *   The above copyright notice and this permission notice shall be included in
 *   all copies or substantial portions of the Software.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 *   BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 *   ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 *   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *   THE SOFTWARE.
 *
 */

package com.owncloud.android.lib.resources.shares

import android.util.Xml

import com.owncloud.android.lib.common.network.WebdavUtils
import com.owncloud.android.lib.resources.files.FileUtils

import org.xmlpull.v1.XmlPullParser
import org.xmlpull.v1.XmlPullParserException
import org.xmlpull.v1.XmlPullParserFactory

import java.io.IOException
import java.io.InputStream
import java.util.ArrayList

/**
 * Parser for Share API Response
 *
 * @author masensio
 * @author David González Verdugo
 */

class ShareXMLParser {

    // Getters and Setters
    var status: String? = null
    var statusCode: Int = 0
    var message: String? = null

    val isSuccess: Boolean
        get() = statusCode == SUCCESS

    val isForbidden: Boolean
        get() = statusCode == ERROR_FORBIDDEN

    val isNotFound: Boolean
        get() = statusCode == ERROR_NOT_FOUND

    val isWrongParameter: Boolean
        get() = statusCode == ERROR_WRONG_PARAMETER

    // Constructor
    init {
        statusCode = -1
    }

    /**
     * Parse is as response of Share API
     * @param is
     * @return List of ShareRemoteFiles
     * @throws XmlPullParserException
     * @throws IOException
     */
    @Throws(XmlPullParserException::class, IOException::class)
    fun parseXMLResponse(`is`: InputStream): ArrayList<RemoteShare> {

        try {
            // XMLPullParser
            val factory = XmlPullParserFactory.newInstance()
            factory.isNamespaceAware = true

            val parser = Xml.newPullParser()
            parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false)
            parser.setInput(`is`, null)
            parser.nextTag()
            return readOCS(parser)

        } finally {
            `is`.close()
        }
    }

    /**
     * Parse OCS node
     * @param parser
     * @return List of ShareRemoteFiles
     * @throws XmlPullParserException
     * @throws IOException
     */
    @Throws(XmlPullParserException::class, IOException::class)
    private fun readOCS(parser: XmlPullParser): ArrayList<RemoteShare> {
        var shares = ArrayList<RemoteShare>()
        parser.require(XmlPullParser.START_TAG, ns, NODE_OCS)
        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.eventType != XmlPullParser.START_TAG) {
                continue
            }
            val name = parser.name
            // read NODE_META and NODE_DATA
            if (name.equals(NODE_META, ignoreCase = true)) {
                readMeta(parser)
            } else if (name.equals(NODE_DATA, ignoreCase = true)) {
                shares = readData(parser)
            } else {
                skip(parser)
            }

        }
        return shares
    }

    /**
     * Parse Meta node
     * @param parser
     * @throws XmlPullParserException
     * @throws IOException
     */
    @Throws(XmlPullParserException::class, IOException::class)
    private fun readMeta(parser: XmlPullParser) {
        parser.require(XmlPullParser.START_TAG, ns, NODE_META)
        //Log_OC.d(TAG, "---- NODE META ---");
        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.eventType != XmlPullParser.START_TAG) {
                continue
            }
            val name = parser.name

            if (name.equals(NODE_STATUS, ignoreCase = true)) {
                status = readNode(parser, NODE_STATUS)

            } else if (name.equals(NODE_STATUS_CODE, ignoreCase = true)) {
                statusCode = Integer.parseInt(readNode(parser, NODE_STATUS_CODE))

            } else if (name.equals(NODE_MESSAGE, ignoreCase = true)) {
                message = readNode(parser, NODE_MESSAGE)

            } else {
                skip(parser)
            }

        }
    }

    /**
     * Parse Data node
     * @param parser
     * @return
     * @throws XmlPullParserException
     * @throws IOException
     */
    @Throws(XmlPullParserException::class, IOException::class)
    private fun readData(parser: XmlPullParser): ArrayList<RemoteShare> {
        val shares = ArrayList<RemoteShare>()
        var share: RemoteShare? = null

        parser.require(XmlPullParser.START_TAG, ns, NODE_DATA)
        //Log_OC.d(TAG, "---- NODE DATA ---");
        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.eventType != XmlPullParser.START_TAG) {
                continue
            }
            val name = parser.name
            if (name.equals(NODE_ELEMENT, ignoreCase = true)) {
                readElement(parser, shares)

            } else if (name.equals(NODE_ID, ignoreCase = true)) {// Parse Create XML Response
                share = RemoteShare()
                val value = readNode(parser, NODE_ID)
                share.setIdRemoteShared(Integer.parseInt(value).toLong())

            } else if (name.equals(NODE_URL, ignoreCase = true)) {
                // NOTE: this field is received in all the public shares from OC 9.0.0
                // in previous versions, it's received in the result of POST requests, but not
                // in GET requests
                share!!.shareType = ShareType.PUBLIC_LINK
                val value = readNode(parser, NODE_URL)
                share.shareLink = value

            } else if (name.equals(NODE_TOKEN, ignoreCase = true)) {
                share!!.token = readNode(parser, NODE_TOKEN)

            } else {
                skip(parser)

            }
        }

        if (share != null) {
            // this is the response of a request for creation; don't pass to isValidShare()
            shares.add(share)
        }

        return shares

    }

    /**
     * Parse Element node
     * @param parser
     * @return
     * @throws XmlPullParserException
     * @throws IOException
     */
    @Throws(XmlPullParserException::class, IOException::class)
    private fun readElement(parser: XmlPullParser, shares: ArrayList<RemoteShare>) {
        parser.require(XmlPullParser.START_TAG, ns, NODE_ELEMENT)

        val share = RemoteShare()

        //Log_OC.d(TAG, "---- NODE ELEMENT ---");
        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.eventType != XmlPullParser.START_TAG) {
                continue
            }

            val name = parser.name

            if (name.equals(NODE_ELEMENT, ignoreCase = true)) {
                // patch to work around servers responding with extra <element> surrounding all
                // the shares on the same file before
                // https://github.com/owncloud/core/issues/6992 was fixed
                readElement(parser, shares)

            } else if (name.equals(NODE_ID, ignoreCase = true)) {
                share.setIdRemoteShared(Integer.parseInt(readNode(parser, NODE_ID)).toLong())

            } else if (name.equals(NODE_ITEM_TYPE, ignoreCase = true)) {
                share.isFolder = readNode(parser, NODE_ITEM_TYPE).equals(TYPE_FOLDER, ignoreCase = true)
                fixPathForFolder(share)

            } else if (name.equals(NODE_ITEM_SOURCE, ignoreCase = true)) {
                share.itemSource = java.lang.Long.parseLong(readNode(parser, NODE_ITEM_SOURCE))

            } else if (name.equals(NODE_PARENT, ignoreCase = true)) {
                readNode(parser, NODE_PARENT)

            } else if (name.equals(NODE_SHARE_TYPE, ignoreCase = true)) {
                val value = Integer.parseInt(readNode(parser, NODE_SHARE_TYPE))
                share.shareType = ShareType.fromValue(value)

            } else if (name.equals(NODE_SHARE_WITH, ignoreCase = true)) {
                share.shareWith = readNode(parser, NODE_SHARE_WITH)

            } else if (name.equals(NODE_FILE_SOURCE, ignoreCase = true)) {
                share.fileSource = java.lang.Long.parseLong(readNode(parser, NODE_FILE_SOURCE))

            } else if (name.equals(NODE_PATH, ignoreCase = true)) {
                share.path = readNode(parser, NODE_PATH)
                fixPathForFolder(share)

            } else if (name.equals(NODE_PERMISSIONS, ignoreCase = true)) {
                share.permissions = Integer.parseInt(readNode(parser, NODE_PERMISSIONS))

            } else if (name.equals(NODE_STIME, ignoreCase = true)) {
                share.sharedDate = java.lang.Long.parseLong(readNode(parser, NODE_STIME))

            } else if (name.equals(NODE_EXPIRATION, ignoreCase = true)) {
                val value = readNode(parser, NODE_EXPIRATION)
                if (value.length != 0) {
                    share.expirationDate = WebdavUtils.parseResponseDate(value)!!.time
                }

            } else if (name.equals(NODE_TOKEN, ignoreCase = true)) {
                share.token = readNode(parser, NODE_TOKEN)

            } else if (name.equals(NODE_STORAGE, ignoreCase = true)) {
                readNode(parser, NODE_STORAGE)
            } else if (name.equals(NODE_MAIL_SEND, ignoreCase = true)) {
                readNode(parser, NODE_MAIL_SEND)

            } else if (name.equals(NODE_SHARE_WITH_DISPLAY_NAME, ignoreCase = true)) {
                share.sharedWithDisplayName = readNode(parser, NODE_SHARE_WITH_DISPLAY_NAME)

            } else if (name.equals(NODE_URL, ignoreCase = true)) {
                val value = readNode(parser, NODE_URL)
                share.shareLink = value

            } else if (name.equals(NODE_NAME, ignoreCase = true)) {
                share.name = readNode(parser, NODE_NAME)

            } else {
                skip(parser)
            }
        }

        if (isValidShare(share)) {
            shares.add(share)
        }
    }

    private fun isValidShare(share: RemoteShare): Boolean {
        return share.remoteId > -1
    }

    private fun fixPathForFolder(share: RemoteShare) {
        if (share.isFolder && share.path != null && share.path!!.length > 0 &&
            !share.path!!.endsWith(FileUtils.PATH_SEPARATOR)
        ) {
            share.path = share.path!! + FileUtils.PATH_SEPARATOR
        }
    }

    /**
     * Parse a node, to obtain its text. Needs readText method
     * @param parser
     * @param node
     * @return Text of the node
     * @throws XmlPullParserException
     * @throws IOException
     */
    @Throws(XmlPullParserException::class, IOException::class)
    private fun readNode(parser: XmlPullParser, node: String): String {
        parser.require(XmlPullParser.START_TAG, ns, node)
        val value = readText(parser)
        //Log_OC.d(TAG, "node= " + node + ", value= " + value);
        parser.require(XmlPullParser.END_TAG, ns, node)
        return value
    }

    /**
     * Read the text from a node
     * @param parser
     * @return Text of the node
     * @throws IOException
     * @throws XmlPullParserException
     */
    @Throws(IOException::class, XmlPullParserException::class)
    private fun readText(parser: XmlPullParser): String {
        var result = ""
        if (parser.next() == XmlPullParser.TEXT) {
            result = parser.text
            parser.nextTag()
        }
        return result
    }

    /**
     * Skip tags in parser procedure
     * @param parser
     * @throws XmlPullParserException
     * @throws IOException
     */
    @Throws(XmlPullParserException::class, IOException::class)
    private fun skip(parser: XmlPullParser) {
        if (parser.eventType != XmlPullParser.START_TAG) {
            throw IllegalStateException()
        }
        var depth = 1
        while (depth != 0) {
            when (parser.next()) {
                XmlPullParser.END_TAG -> depth--
                XmlPullParser.START_TAG -> depth++
            }
        }
    }

    companion object {

        //private static final String TAG = ShareXMLParser.class.getSimpleName();

        // No namespaces
        private val ns: String? = null

        // NODES for XML Parser
        private val NODE_OCS = "ocs"

        private val NODE_META = "meta"
        private val NODE_STATUS = "status"
        private val NODE_STATUS_CODE = "statuscode"
        private val NODE_MESSAGE = "message"

        private val NODE_DATA = "data"
        private val NODE_ELEMENT = "element"
        private val NODE_ID = "id"
        private val NODE_ITEM_TYPE = "item_type"
        private val NODE_ITEM_SOURCE = "item_source"
        private val NODE_PARENT = "parent"
        private val NODE_SHARE_TYPE = "share_type"
        private val NODE_SHARE_WITH = "share_with"
        private val NODE_FILE_SOURCE = "file_source"
        private val NODE_PATH = "path"
        private val NODE_PERMISSIONS = "permissions"
        private val NODE_STIME = "stime"
        private val NODE_EXPIRATION = "expiration"
        private val NODE_TOKEN = "token"
        private val NODE_STORAGE = "storage"
        private val NODE_MAIL_SEND = "mail_send"
        private val NODE_SHARE_WITH_DISPLAY_NAME = "share_with_displayname"
        private val NODE_SHARE_WITH_ADDITIONAL_INFO = "share_with_additional_info"
        private val NODE_NAME = "name"

        private val NODE_URL = "url"

        private val TYPE_FOLDER = "folder"

        private val SUCCESS = 200
        private val ERROR_WRONG_PARAMETER = 400
        private val ERROR_FORBIDDEN = 403
        private val ERROR_NOT_FOUND = 404
    }
}